ASM
ORG $200
Main

	; **** INITIALIZATION *************************************************

	; Set RA ports to output
 	_MODE	#$1F
	MOV	RA,  #%00000000
	MOV	!RA, #%00000000
	MOV	RC,  #( MASK_SRAM_ADDR_CLOCK_OFF | MASK_SRAM_ADDR_DATA_0 | MASK_SRAM_DISABLE | MASK_SRAM_DIR_WRITE )
	MOV	!RC, #%00000000

	; Set RE ports to output
	MOV	RE,  #%00000000
	MOV	!RE, #%00000000

	_BANK	BANK_VIDEO

	; Set the default color phase
	MOV	color_phase, #COLOR_14 - 1

	; Set the visible & active pages to the bottom of SRAM
	SET_VISIBLE_PAGE #0, #0
	SET_ACTIVE_PAGE	 #0, #0

	; Initialize the game
	_BANK	BANK_SXB
	CALL	@__Game_Init

	; **** DRAW SCREEN ****************************************************

	_BANK	BANK_VIDEO

:draw_screen

	; Reset the tile row
	CLR	tile_row

	; Reset the row page pointer
	MOV	fb_ptr_lo, visible_page_lo
	MOV	fb_ptr_hi, visible_page_hi

	; **** DRAW ACTIVE SCANLINES ******************************************

	MOV	scan_count, #192
:draw_active_scanline

	; **** INITIALIZE SCANLINE AND READ DATA ******************************

	MOV	RE, #BLACK		; Front porch - 1.5us
	DELAY	( CLK_SCALE * 15 - 2 )

	; **** H-SYNC (376 CLOCKS) ********************************************

	MOV	RE, #SYNC		; (2) Emit sync for H-sync period

	MOV	t0, fb_ptr_lo		; (1)   Copy the framebuffer pointer into <t1:t0>
	MOV	t1, fb_ptr_hi		; (1)
	CALL	@__SRAM_Load_Page_R	; (139) Load the proper SRAM page

	; **** Read first half of tile row data

	MOV	t0, #8			; (2) Read 8 tiles
	MOV	t1, tile_row		; (2) The current tile row to read
	CLR	FSR			; (1)
	_BANK	BANK_TILE_ROW_BUFF	; (2) Switch to the tile row data bank	
:read_tile

	; Read the next tile index from SRAM
	MOV	t2, RD			; (2)
	CLC				; (1) Multiply the tile index by 8
	RL	t2			; (1)
	RL	t2			; (1)
	RL	t2			; (1)

	; Read the next 8-bit tile row data
	MOV 	t3, #tiles >> 8		; (2) Build the high nibble of the address in <t3>
	ADDB	t3, RD.5		; (2) Add the high bit of the index for tiles 32-63
	MOV	M, t3			; (1)
	MOV	W, #tiles & $FF		; (1)
	ADD	W, t1			; (1)
	ADD	W, t2			; (1) Add the tile index
	IREAD				; (4)
	MOV	IND, W			; (1) Write the row data
	ADD	FSR, #3			; (2) Move to the next character

	INC	RC			; (1) Move to the next SRAM byte

	; Move to the next byte in the SRAM page and loop	
	DJNZ	t0, :read_tile		; (2/4)

	_BANK	BANK_VIDEO		; (2) Restore the video bank

	; Read Loop: 220 Clocks
	
	DELAY	( 376 - 220 - 142 - 2 - 11 ) ; **** Fudge factor (fix later)

	; **** FINISH SCANLINE INITIALIZATION *********************************

	MOV	RE, #BLACK		; Pre-burst - .6us
	DELAY	( CLK_SCALE * 6 - 2 )
	MOV	RE, color_phase		; Color-burst reference - 2.5us
	DELAY	( CLK_SCALE * 25 - 2 )
	MOV	RE, #BLACK		; Post-burst - 1.6us
	DELAY	( CLK_SCALE * 16 - 2 )

	; **** DRAW SCREEN (4208 CLOCKS) **************************************

	; **** DRAW LEFT-HAND BORDER ******************************************

	MOV	RE, #BLACK		; (2)

	; **** Read second half of tile row data

	; Get the current tile row to read
	_BANK	BANK_VIDEO		; (2)
	MOV	t1, tile_row		; (2)

	; Set up the loop
	MOV	t0, #8			; (2) Read 8 tiles
	CLR	FSR			; (1)
	_BANK	BANK_TILE_ROW_BUFF	; (2) Switch to the tile row data bank
	ADD	FSR, #24		; (2) Start on the 8th 3-byte tile structure
:read_tile_2

	; Read the next tile index from SRAM
	MOV	t2, RD			; (2)
	CLC				; (1) Multiply the tile index by 8
	RL	t2			; (1)
	RL	t2			; (1)
	RL	t2			; (1)

	; Read the next 8-bit tile row data
	MOV 	t3, #tiles >> 8		; (2) Build the high nibble of the address in <t3>
	ADDB	t3, RD.5		; (2) Add the high bit of the index for tiles 32-63
	MOV	M, t3			; (1)
	MOV	W, #tiles & $FF		; (1)
	ADD	W, t1			; (1)
	ADD	W, t2			; (1) Add the tile index
	IREAD				; (4)
	MOV	IND, W			; (1) Write the row data
	ADD	FSR, #3			; (1) Move to the next character
	INC	RC			; (1) Move to the next byte

	; Move to the next byte in the SRAM page and loop	
	DJNZ	t0, :read_tile_2	; (2/4)
	; Loop total: 208 clocks

	; Read tiles total: 217 clocks

	; **** Read the row's foreground and background colors

	; Load the foreground color page

	; Load the SRAM Page
	_BANK	BANK_VIDEO		; (2)
	MOV	t0, fb_ptr_lo		; (1)   Copy the framebuffer pointer into <t1:t0>
	MOV	t1, fb_ptr_hi		; (1)
	ADD	t0, #1			; (2)   Move to the second page
	ADDB	t1, C			; (2)
	CALL	@__SRAM_Load_Page_R	; (139)
	; 146 clocks

	; Copy the page into RAM
	MOV	t0, #4			; (2)
	CLR	FSR			; (1)
	_BANK	BANK_TILE_ROW_BUFF	; (2) Switch to the tile row data bank
	INC	FSR			; (1) Offset by 1 the for foreground color field
:read_fg_color
	REPT	4
	MOV	IND, RD			; (2)
	INC	RC			; (1) Move to the next tile
	ADD	FSR, #3			; (2)
	ENDR
	DJNZ	t0, :read_fg_color	; (2/4)
	; Read FG colors: 100 + 146 = 246 clocks

	; Load the background color page

	; Load the SRAM Page
	_BANK	BANK_VIDEO		; (2)
	MOV	t0, fb_ptr_lo		; (1) Copy the framebuffer pointer into <t1:t0>
	MOV	t1, fb_ptr_hi		; (1)
	ADD	t0, #2			; (2) Move to the third page
	ADDB	t1, C			; (2)
	CALL	@__SRAM_Load_Page_R	; (138)

	; Copy the page into RAM
	MOV	t0, #4			; (2)
	CLR	FSR			; (1)
	_BANK	BANK_TILE_ROW_BUFF	; (2) Switch to the tile row data bank
	ADD	FSR, #2			; (2) Offset by 2 the for background color field
:read_bg_color
	REPT	4
	MOV	IND, RD			; (2)
	INC	RC			; (1) Move to the next tile
	ADD	FSR, #3			; (2)
	ENDR
	DJNZ	t0, :read_bg_color	; (2/4)
	; Read FG colors: 101 + 146 = 247 clocks

	SRAM_OFF			; (1) Disable SRAM
	_BANK	BANK_VIDEO		; (2) Restore the video bank

	; Left-hand border processing total: 710 + 3 = 713 clocks

	; **** DRAW TILES *****************************************************

	;  Switch to the tile row buffer bank
	CLR	FSR			; (1)
	_BANK	BANK_TILE_ROW_BUFF	; (2)

	; Draw the next tile row (8 pixels)
	MOV	t2, #16			; (2) Draw scanline in tile row increments (16 tiles, 8 pixels each)
:draw_tile_row

	; **** READ TILE ROW **************************************************
	
	MOV	t5, IND			; (2) Read the tile data
	INC	FSR			; (1) Point to the color value
	MOV	t3, IND			; (2) Read the color data
	INC	FSR			; (1) Point to the next tile/color pair
	MOV	t4, IND			; (2) Read the color data
	INC	FSR			; (1) Point to the next tile/color pair
	; Total: 9 clocks

	; **** DRAW FIRST PIXEL ***********************************************

	; Draw the first pixel outside of the row loop to absorb the timing
	; offset created above when reading the tile row data
	
	; Read the first pixel (bit 7)
	MOV	W, t4			; (1)   Draw with the foreground color
	RL	t5			; (1)   Shift out bit 7
	SNC				; (1/2)
	MOV	W, t3			; (1)   Draw with the background color
	MOV	RE, W			; (1)   Set the output color

	NOP
	NOP
	NOP
	NOP
	NOP
	; Total: 10 clocks

	; **** DRAW REMAINING PIXELS ******************************************

	REPT	7

	NOP4 				; (4)   Leading pixel padding
	NOP4 				; (4)
	NOP				; (1)

	MOV	W, t4			; (1)   Draw with the foreground color
	RL	t5			; (1)   Shift out bit 7
	SNC				; (1/2)
	MOV	W, t3			; (1)   Draw with the background color
	MOV	RE, W			; (1)   Set the output color

	NOP4				; (4)   Trailing pixel padding.
	NOP4 				; (4)
	NOP				; (1)

	ENDR
	; Total: 23 * 7 = 161 clocks

	DJNZ	t2, :draw_tile_row	; (2/4) Draw the next tile row
	; Loop total: 9 + 10 + 161 + 4 = 184 * 16 = 2944 clocks

	; **** DRAW RIGHT-HAND BORDER *****************************************

	MOV	RE, #BLACK
	DELAY	( 588 - 18 - 2 - 18 - 40 )	; Subtract tile update code time below
						; **** Fudge factor (fix later)
						; v1.1 (12.6.2004) -40 clocks for 64 tile access

	; **** UPDATE TILE POINTERS *******************************************

	_BANK	BANK_VIDEO		; (2) Switch back to the video bank

	; Increment the tile base row address
	INC	tile_row		; (1)
	AND	tile_row, #7		; (2)

	; Increment the framebuffer pointer if the tile is complete
	CJNE	tile_row, #0, :skip_new_row	; (4/6)
	NOP2					; (2)
	ADD	fb_ptr_lo, #4			; (2) Move ahead 4 pages
	ADDB	fb_ptr_hi, C			; (2)
	JMP	:new_row_done			; (3)
:skip_new_row
	NOP2					; (2)
	NOP2					; (2)
	JMP	$ + 1				; (3)
:new_row_done

	; Total: 18 clocks

	; **** END OF RASTER, NEXT SCANLINE ***********************************

	_BANK	BANK_VIDEO
	DJNZ	scan_count, :draw_active_scanline

	; **** BOTTOM SCREEN OVERSCAN *****************************************

	MOV	scan_count, #24
:draw_bottom_overscan
	CALL	@__Init_Scanline
	MOV 	RE, #OVERSCAN_COLOR
	DELAY	( CLK_SCALE * 526 - 2 - 4 - 2 )
	_BANK	BANK_VIDEO
	DJNZ	scan_count, :draw_bottom_overscan

	; **** VERTICAL SYNC (~60,690 CLOCKS) *********************************

	; Emit sync during V-sync
	MOV	RE, #SYNC

	; Reset RTCC and set prescaler to 1:256, allowing the 8-bit counter to
	; keep [very] approximate time over 60,690 clocks.
	MOV	!OPTION, #%11000111		; Turn off interrupts and set prescaler
	CLR	RTCC

	; Handle game logic
	_BANK	BANK_SXB
	CALL	@__Game_Update
	_BANK	BANK_VIDEO			; Make sure the proper bank is selected upon return

	; Delay for remainder of V-Sync
	CJB	RTCC, #237, $			; Closest multiple of 256 to 60,690 is
						; 60,672 (256 * 237). Loop until the RTCC
						; Reaches this point.
	DELAY	( 60_690 - 60_672 )		; Pad difference between actual V-sync
						; length and prescaler approximation

	; **** TOP SCREEN OVERSCAN ********************************************

	MOV	scan_count, #34
:draw_top_overscan
	CALL	@__Init_Scanline
	MOV	RE, #OVERSCAN_COLOR
	DELAY	( CLK_SCALE * 526 - 2 - 4 - 2 )
	_BANK	BANK_VIDEO
	DJNZ	scan_count, :draw_top_overscan

	; **** DRAW NEXT FRAME ************************************************

	JMP	:draw_screen

End_Main

; **** SUBROUTINES ************************************************************

ORG $400

; *****************************************************************************
; *
; *	Init_Scanline ()
; *
; *	Sets up scanline rasterization. WARNING: CALLED BY MAIN VIDEO LOOP

Init_Scanline

	MOV	RE, #BLACK		; Front porch - 1.5us
	DELAY	( CLK_SCALE * 15 - 2 )
	MOV	RE, #SYNC		; H-sync - 4.7us
	DELAY	( CLK_SCALE * 47 - 2 )
	MOV	RE, #BLACK		; Pre-burst - .6us
	DELAY	( CLK_SCALE * 6 - 2 )
	MOV	RE, color_phase		; Color-burst reference - 2.5us
	DELAY	( CLK_SCALE * 25 - 2 )
	MOV	RE, #BLACK		; Post-burst - 1.6us
	DELAY	( CLK_SCALE * 16 - 2 )
	RETP

End_Init_Scanline

; *****************************************************************************
; *
; *	SRAM_Write ()
; *
; *	Writes a value to a random address in SRAM.
; *
; *	INPUTS
; *		t1:t0 - 16-bit address
; *		t2    - Data

SRAM_Write

	; Work in the low page
	CLRB	RA.7

	; Set lower nibble of <W> with 4-bit page offset, upper nibble of <W> with
	; SRAM control lines
	MOV	W, t0
	AND	W, #%00001111
	OR	W, #( MASK_SRAM_ADDR_CLOCK_OFF | MASK_SRAM_ADDR_DATA_0 | MASK_SRAM_DISABLE | MASK_SRAM_DIR_WRITE )
	MOV	RC, W

	; Set data bus to output
	_MODE	#$1F
	MOV	!RD, #%00000000
	MOV	RD, t2			; Put data on bus

	; Shift 12-bit page base address in
	MOV	t2, #3			; Shift 12 bits (loop unrolled 4x)
:shift_addr

	REPT	4
	CLRB	RC.BIT_SRAM_ADDR_CLOCK	; Turn the clock low
	NOP
	CLRB	RC.BIT_SRAM_ADDR_DATA	; Assume address bit is low
	SNB	t1.7			; Set address data bit high if address bit 11 is high
	SETB	RC.BIT_SRAM_ADDR_DATA
	NOP				; Delay for setup time
	SETB	RC.BIT_SRAM_ADDR_CLOCK	; Turn the clock hi
	RL	t0			; Shift the 16-bit address left for the next address bit
	RL	t1
	ENDR

	DJNZ	t2, :shift_addr

	; Write the data
	CLRB	RC.BIT_SRAM_ADDR_CLOCK	; Clear the SRAM clock line
	CLRB	RC.BIT_SRAM_DIR_nWE	; Set SRAM write mode
	CLRB	RC.BIT_SRAM_nOE		; Enable SRAM

	NOP2				; Hold

	SRAM_OFF			; Disable SRAM

	RETP

End_SRAM_Write

; *****************************************************************************
; *
; *	SRAM_Read ()
; *
; *	Reads a value from a random address in SRAM.
; *
; *	INPUTS
; *		t1:t0 - 16-bit address
; *
; *	OUTPUTS
; *		t0    - Data

SRAM_Read

	; Work in the low page
	CLRB	RA.7

	; Set lower nibble of <W> with 4-bit page offset, upper nibble of <W> with
	; SRAM control lines
	MOV	W, t0
	AND	W, #%00001111
	OR	W, #( MASK_SRAM_ADDR_CLOCK_OFF | MASK_SRAM_ADDR_DATA_0 | MASK_SRAM_DISABLE | MASK_SRAM_DIR_READ )
	MOV	RC, W

	; Set data bus to input
	_MODE	#$1F
	CLR	RD
	MOV	!RD, #%11111111

	; Shift 12-bit page base address in
	MOV	t2, #3			; Shift 12 bits (unrolled 4x)
:shift_addr

	REPT	4
	CLRB	RC.BIT_SRAM_ADDR_CLOCK	; Turn the clock low
	NOP
	CLRB	RC.BIT_SRAM_ADDR_DATA	; Assume address bit is low
	SNB	t1.7			; Set address data bit high if address bit 11 is high
	SETB	RC.BIT_SRAM_ADDR_DATA
	NOP				; Delay for setup time
	SETB	RC.BIT_SRAM_ADDR_CLOCK	; Turn the clock hi
	RL	t0			; Shift the 16-bit address left for the next address bit
	RL	t1
	ENDR
	
	DJNZ	t2, :shift_addr

	; Read the data
	CLRB	RC.BIT_SRAM_ADDR_CLOCK	; Clear the SRAM clock line
	SETB	RC.BIT_SRAM_DIR_nWE	; Set SRAM read mode
	CLRB	RC.BIT_SRAM_nOE		; Enable SRAM
	NOP2				; Hold

	MOV	t0, RD			; Read the data from the data bus
	NOP2				; Hold

	SRAM_OFF			; Disable SRAM

	RETP

End_SRAM_Read

; *****************************************************************************
; *
; *	SRAM_Load_Page_R ()
; *
; *	Prepares an SRAM page for reading. WARNING: CALLED BY MAIN VIDEO LOOP
; *
; *	INPUTS
; *		t1:t0 - Page base address (lower nibble of <t1> used)

SRAM_Load_Page_R

	; Work in the low page
	CLRB	RA.7			; (1)

	; Set lower nibble of <W> with 4-bit page offset, upper nibble of <W> with
	; SRAM control lines
	CLR	W												  ; (1)
	OR	W, #( MASK_SRAM_ADDR_CLOCK_OFF | MASK_SRAM_ADDR_DATA_0 | MASK_SRAM_DISABLE | MASK_SRAM_DIR_READ ) ; (1)
	MOV	RC, W												  ; (1)

	; Set data bus to input
	_MODE	#$1F			; (2)
	CLR	RD			; (1)
	MOV	!RD, #%11111111		; (2)

	; Shift 12-bit page base address in
	MOV	t2, #3			; (2) Shift 12 bits (loop unrolled 4x)
:shift_addr
	REPT	4
	CLRB	RC.BIT_SRAM_ADDR_CLOCK	; (1)   Turn the clock low
	NOP				; (1)
	CLRB	RC.BIT_SRAM_ADDR_DATA	; (1)   Assume address bit is low
	SNB	t1.3			; (1/2) Set address data bit high if address bit 11 is high
	SETB	RC.BIT_SRAM_ADDR_DATA	; (1)
	NOP				; (1)   Delay for setup time
	SETB	RC.BIT_SRAM_ADDR_CLOCK	; (1)   Turn the clock hi
	RL	t0			; (1)   Shift the 16-bit address left for the next address bit
	RL	t1			; (1)
	ENDR
	DJNZ	t2, :shift_addr		; (2/4)
	; Loop total: 120 clocks

	; Prepare the SRAM for reading
	CLRB	RC.BIT_SRAM_ADDR_CLOCK	; (1) Clear the SRAM clock line
	SETB	RC.BIT_SRAM_DIR_nWE	; (1) Set SRAM read mode
	CLRB	RC.BIT_SRAM_nOE		; (1) Enable SRAM

	; SRAM Setup total = 132 clocks (Plus CALL @ & RETP = 132 + 7 = 139)
	RETP

End_SRAM_Load_Page_R

; *****************************************************************************
; *
; *	SRAM_Load_Page_W ()
; *
; *	Prepares an SRAM page for writing.
; *
; *	INPUTS
; *		t1:t0 - Page base address (lower nibble of <t0> used)

SRAM_Load_Page_W

	; Work in the low page
	CLRB	RA.7			; (1)

	; Set data bus to output
	_MODE	#$1F			; (2)
	MOV	!RD, #%00000000		; (2)

	; Set lower nibble of <W> with 4-bit page offset, upper nibble of <W> with
	; SRAM control lines
	CLR	W												   ; (1)
	OR	W, #( MASK_SRAM_ADDR_CLOCK_OFF | MASK_SRAM_ADDR_DATA_0 | MASK_SRAM_DISABLE | MASK_SRAM_DIR_WRITE ) ; (1)
	MOV	RC, W												   ; (1)

	; Shift 12-bit page base address in
	MOV	t2, #2			; (2) Shift 12 bits (loop unrolled 6x)
:shift_addr
	REPT	6
	CLRB	RC.BIT_SRAM_ADDR_CLOCK	; (1)   Turn the clock low
	NOP				; (1)
	CLRB	RC.BIT_SRAM_ADDR_DATA	; (1)   Assume address bit is low
	SNB	t1.3			; (1/2) Set address data bit high if address bit 11 is high
	SETB	RC.BIT_SRAM_ADDR_DATA	; (1)
	NOP				; (1)   Delay for setup time
	SETB	RC.BIT_SRAM_ADDR_CLOCK	; (1)   Turn the clock hi
	RL	t0			; (1)   Shift the 16-bit address left for the next address bit
	RL	t1			; (1)
	ENDR
	DJNZ	t2, @:shift_addr		; (2/4)
	; Address shift loop total = ( 36 + 4 ) * 3 = 120

	; Prepare the SRAM for writing
	CLRB	RC.BIT_SRAM_ADDR_CLOCK	; (1) Clear the SRAM clock line
	CLRB	RC.BIT_SRAM_DIR_nWE	; (1) Set SRAM write mode
	CLRB	RC.BIT_SRAM_nOE		; (1) Enable SRAM

	; SRAM Setup total = 120 + 10 = 130 clocks (Plus CALL @ & RETP = 130 + 7 = 137)
	RETP

End_SRAM_Load_Page_W

ENDASM

' *****************************************************************************
' *
' *	Set_Tile (X, Y, TILE_INDEX)
' *
' *	Sets a specific tile's index and color attributes.
' *
' *	INPUTS
' *		t4	   - X
' *		t5	   - Y
' *		tile_index - Tile index
' *		fg_color   - Foreground color
' *		bg_color   - Background color
Set_Tile:
	BANK BANK_VIDEO
	t4 = __PARAM1
	t5 = __PARAM2	
	tile_index = __PARAM3
ASM
Set_Tile_mac
	
	; **** BOUNDS CHECK ***************************************************

	; Make sure tile is within 0-15, 0-23
	CJA	t4, #15, :no_draw
	CJA	t5, #23, :no_draw

	; **** SET TILE INDEX *************************************************

	; Set <t1:t0> to the value of <t5>, then shift left 2 bits to multiply
	; by 4 to get the base address of the page. Finally, shift left 4 more
	; bits to place the value in the page nibble, allowing the page offset
	; nibble to be set next
	MOV	t0, t5
	CLR	t1
	CLC
	REPT	2 + 4
	RL	t0
	RL	t1
	ENDR

	; Add the X location stored in <t4> to the address in <t1:t0>
	ADD	t0, t4
	ADDB	t1, C

	_BANK	BANK_VIDEO

	; Add the active page offset
	ADD	t0, active_page_lo
	ADDB	t1, C
	ADD	t1, active_page_hi

	; Copy the completed address into <t5:t4> for later use
	MOV	t4, t0
	MOV	t5, t1

	; Write the tile index
	MOV	t2, tile_index
	CALL	@__SRAM_Write

	; **** SET FOREGROUND COLOR *******************************************

	; Restore the address in <t1:t0>
	MOV	t0, t4
	MOV	t1, t5

	; Shift the address over to the next page
	ADD	t0, #16
	ADDB	t1, C

	; Write the foreground color
	_BANK	BANK_VIDEO
	MOV	t2, fg_color
	CALL	@__SRAM_Write

	; **** SET BACKGROUND COLOR *******************************************

	; Restore the address in <t1:t0>
	MOV	t0, t4
	MOV	t1, t5

	; Shift the address over to the next page
	ADD	t0, #32
	ADDB	t1, C

	; Write the foreground color
	_BANK	BANK_VIDEO
	MOV	t2, bg_color
	CALL	@__SRAM_Write

:no_draw
ENDASM
	BANK BANK_SXB
	RETURN


' *****************************************************************************
' *
' *	Set_Tile_FG (X, Y, TILE_INDEX)
' *
' *	Sets a specific tile's index and foreground color, leaving the
' *	background color intact.
' *
' *	INPUTS
' *		t4	   - X
' *		t5	   - Y
' *		tile_index - Tile index
' *		fg_color   - Foreground color

Set_Tile_FG:
	BANK BANK_VIDEO
	t4 = __PARAM1
	t5 = __PARAM2	
	tile_index = __PARAM3
	
ASM
Set_Tile_FG_mac:
	
	; **** BOUNDS CHECK ***************************************************

	; Make sure tile is within 0-15, 0-23
	CJA	t4, #15, :no_draw
	CJA	t5, #23, :no_draw

	; **** SET TILE INDEX *************************************************

	; Set <t1:t0> to the value of <t5>, then shift left 2 bits to multiply
	; by 4 to get the base address of the page. Finally, shift left 4 more
	; bits to place the value in the page nibble, allowing the page offset
	; nibble to be set next
	MOV	t0, t5
	CLR	t1
	CLC
	REPT	2 + 4
	RL	t0
	RL	t1
	ENDR

	; Add the X location stored in <t4> to the address in <t1:t0>
	ADD	t0, t4
	ADDB	t1, C

	_BANK	BANK_VIDEO

	; Add the active page offset
	ADD	t0, active_page_lo
	ADDB	t1, C
	ADD	t1, active_page_hi

	; Copy the completed address into <t5:t4> for later use
	MOV	t4, t0
	MOV	t5, t1

	; Write the tile index
	MOV	t2, tile_index
	CALL	@__SRAM_Write

	; **** SET FOREGROUND COLOR *******************************************

	; Restore the address in <t1:t0>
	MOV	t0, t4
	MOV	t1, t5

	; Shift the address over to the next page
	ADD	t0, #16
	ADDB	t1, C

	; Write the foreground color
	_BANK	BANK_VIDEO
	MOV	t2, fg_color
	CALL	@__SRAM_Write

:no_draw
ENDASM
	BANK BANK_SXB
	RETURN

' *****************************************************************************
' *
' *	Get_Tile_Index (X, Y, @TILE_INDEX)
' *
' *	Returns the bitmap index of the specified tile.
' *
' *	INPUTS
' *		t4 - X
' *		t5 - Y
' *
' *	OUTPUTS
' *		t0 - Tile index

Get_Tile_Index:
	t4 = __PARAM1
	t5 = __PARAM2
	t2 = __PARAM3
ASM
Get_Tile_Index_Mac

	; **** SET TILE INDEX *************************************************

	; Set <t1:t0> to the value of <t5>, then shift left 2 bits to multiply
	; by 4 to get the base address of the page. Finally, shift left 4 more
	; bits to place the value in the page nibble, allowing the page offset
	; nibble to be set next
	MOV	t0, t5
	CLR	t1
	CLC
	REPT	2 + 4
	RL	t0
	RL	t1
	ENDR

	; Add the X location stored in <t4> to the address in <t1:t0>
	ADD	t0, t4
	ADDB	t1, C

	; Add the active page offset
	ADD	t0, active_page_lo
	ADDB	t1, C
	ADD	t1, active_page_hi

	; Read the tile index
	CALL	@__SRAM_Read
ENDASM
	__RAM(t2)=t0
	RETURN

'ADDRESS $600

' *****************************************************************************
' *
' *	Fill_Row (ROW, TILE_INDEX)
' *
' *	Fills a row with the specified index and foreground/background colors.
' *
' *	INPUTS
' *		t5	   - Row
' *		tile_index - Tile index
' *		fg_color   - Foreground color
' *		bg_color   - Background color
Fill_Row:
	t5 = __PARAM1
	tile_index = __PARAM2
ASM
Fill_Row_mac:
	BANK BANK_VIDEO
	; **** Calculate the 12-bit page address of the row in <t4:t3> ********

	; Set <t4:t3> to the value of <t5>, then shift left 2 bits to multiply
	; by 4 (each row occupies 4 pages)
	MOV	t3, t5
	CLR	t4
	CLC
	REPT	2
	RL	t3
	RL	t4
	ENDR

	; Copy the active page into <t1:t0> and divide it by 16 to convert it to
	; a page offset
	MOV	t1, active_page_hi
	MOV	t0, active_page_lo
	CLC
	REPT	4
	RR	t1
	RR	t0
	ENDR

	; Add the active page offset to the address in <t4:t3>
	ADD	t3, t0
	ADDB	t4, C
	ADD	t4, t1

	; **** Fill the row's tile index field ********************************
	
	; Load the SRAM page base address
	MOV	t0, t3
	MOV	t1, t4
	CALL	@__SRAM_Load_Page_W

	; Get the tile index from the video buffer
	_BANK	BANK_VIDEO
	MOV	t1, tile_index
	
	; Write to each tile
	MOV	t0, #15
:fill_row_tile
	MOV	RD, t1		; Set the tile index
	INC	RC		; Move to the next tile
	DJNZ	t0, :fill_row_tile
	MOV	RD, t1		; Set the tile index

	; **** Fill the row's foreground color field **************************
	
	; Load the SRAM page base address, then move ahead one page
	MOV	t0, t3
	MOV	t1, t4
	ADD	t0, #1
	ADDB	t1, C
	CALL	@__SRAM_Load_Page_W

	; Get the foreground color from the video buffer
	MOV	t1, fg_color
	
	; Write to each tile
	MOV	t0, #15
:fill_row_fg
	MOV	RD, t1		; Set the tile index
	INC	RC		; Move to the next tile
	DJNZ	t0, :fill_row_fg
	MOV	RD, t1		; Set the tile index

	; **** Fill the row's background color field **************************
	
	; Load the SRAM page base address, then move ahead two pages
	MOV	t0, t3
	MOV	t1, t4
	ADD	t0, #2
	ADDB	t1, C
	CALL	@__SRAM_Load_Page_W

	; Get the background color from the video buffer
	MOV	t1, bg_color
	
	; Write to each tile
	MOV	t0, #15
:fill_row_bg
	MOV	RD, t1		; Set the tile index
	INC	RC		; Move to the next tile
	DJNZ	t0, :fill_row_bg
	MOV	RD, t1		; Set the tile index
	
	SRAM_OFF		; Disable SRAM
ENDASM
	BANK BANK_SXB
	RETURN


' *****************************************************************************
' *
' *	Fill_Screen (TILE_INDEX)
' *
' *	Sets each tile to the specified index and foreground/background colors.
' *
' *	INPUTS
' *		tile_index - Tile index
' *		fg_color   - Foreground color
' *		bg_color   - Background color

Fill_Screen:
	tile_index = __PARAM1
ASM
Fill_Screen_mac

	; Fill 24 rows
	MOV	t5, #23
:fill_row
	CALL	@__Fill_Row_Mac 	; Fill the row
	DJNZ	t5, :fill_row
	CALL	@__Fill_Row_Mac 	; Fill the last row
ENDASM
	RETURN

' *****************************************************************************
' *
' *	Load_Map ()
' *
' *	Decodes and loads an encoded, 16x24 tile map into SRAM from program
' *	memory.
' *
' *	INPUTS
' *		t1:t0 - Pointer to source map in program memory
' *		t3:t2 - Pointer to tile attribute table

Load_Map:
ASM
Load_Map_Mac

	; Save the tile attribute table pointer on the stack
	_BANK	BANK_STACK
	MOV	s7, t3
	MOV	s6, t2

	; Decode 24 rows
	CLR	t5			; Row counter
:decode_row

	; Decode 16 tiles (6 program words)
	CLR	t4			; Tile counter
	CLR	t2			; Nibble counter
:decode_tile

	; **** DECODE THE TILE BASED ON THE NIBBLE COUNTER ********************

	; Read nibble 0?
	CJNE	t2, #0, :read_nibble_1
	MOV	M, t1			; Read the program word
	MOV	W, t0
	IREAD
	MOV	t3, M			; Tile is in <M>
	JMP	:tile_decoded

	; Read nibble 1?
:read_nibble_1
	CJNE	t2, #1, :read_nibble_2	; Read the program word
	MOV	M, t1
	MOV	W, t0
	IREAD
	MOV	t3, W			; Tile is in high nibble of <W>
	RR	t3
	RR	t3
	RR	t3
	RR	t3
	AND	t3, #%00001111
	JMP	:tile_decoded

	; Read nibble 2?
:read_nibble_2
	MOV	M, t1
	MOV	W, t0
	IREAD
	MOV	t3, W			; Tile is in low nibble of <W>
	AND	t3, #%00001111
	ADD	t0, #1			; Read next program word
	ADDB	t1, C

	; Tile decoding finished
:tile_decoded

	; Increment the nibble counter
	INC	t2
	CJB	t2, #3, :skip_wrap_nibble
	CLR	t2
:skip_wrap_nibble

	; **** READ THE TILE'S ATTRIBUTES *************************************

	; Save the globals
	PUSH_GLOBALS

	; Multiply the tile index by 4 to get the tile attribute table offset
	_BANK	BANK_STACK
	CLC
	RL	t3
	RL	t3
	ADD	t3, s6			; Add base address of table

	; Load the tile's index
	MOV	M, s7
	MOV	W, t3			; Tile offset + 0
	IREAD
	MOV	t1, W
	INC	t3			; Move to next field

	; Load the tile's foreground color
	MOV	M, s7
	MOV	W, t3			; Tile offset + 1
	IREAD
	MOV	t2, W
	INC	t3			; Move to next field

	; Load the tile's background color
	MOV	M, s7
	MOV	W, t3			; Tile offset + 2
	IREAD
	MOV	t0, W

	; **** WRITE THE TILE TO THE FRAMEBUFFER ******************************

	_BANK	BANK_VIDEO
	M_SET_TILE	t4, t5, t1, t2, t0

	; Restore the globals
	POP_GLOBALS

	; Decode the next tile
	INC	t4
	CJB	t4, #16, :decode_tile

	; Skip the alignment word
	ADD	t0, #1
	ADDB	t1, C

	; Decode the next row
	INC	t5
	CJB	t5, #24, :decode_row
ENDASM
	BANK BANK_SXB
	RETURN

' *****************************************************************************
' *
' *	Copy_Screen (sourceH, sourceL, destH, destL)
' *
' *	Copies a 16x24 framebuffer from one location in SRAM to another. Can
' *	be used to quickly restore backgrounds while moving on-screen objects,
' *	or to change backgrounds (to implement screen scrolling, for example).
' *
' *	INPUTS
' *		t1:t0 - 12-bit base page address of source screen
' *		t3:t2 - 12-bit base page address of destination screen

Copy_Screen:
'	t0 = __PARAM1
'	t1 = __PARAM2
'	t2 = __PARAM3
'	t3 = __PARAM4

ASM
Copy_Screen_mac

	; Copy the screen base addresses onto the stack
	_BANK	BANK_STACK
	MOV	s7, t0
	MOV	s6, t1
	MOV	s9, t2
	MOV	s8, t3

	; Copy 24 rows
	MOV	t0, #24
:copy_row
	
	; Save the row counter (stack bank already selected)
	MOV	s0, t0

	; Copy 3 pages
	MOV	t5, #3
:copy_page
	
	; Load the next source SRAM page to read
	_BANK	BANK_STACK
	MOV	t1, s7			; t1:t0 points to the base of the page
	MOV	t0, s6
	CALL	@__SRAM_Load_Page_R

	; Copy the SRAM page to a RAM buffer
	MOV	t0, #4			; Unroll loop 4x
	CLR	FSR
	_BANK	BANK_SRAM_COPY_BUFF
:sram_to_ram

	MOV	IND, RD			; 0
	INC	FSR
	INC	RC
	NOP
	NOP
	MOV	IND, RD			; 1
	INC	FSR
	INC	RC
	NOP
	NOP
	MOV	IND, RD			; 2
	INC	FSR
	INC	RC
	NOP
	NOP
	MOV	IND, RD			; 3
	INC	FSR
	INC	RC

	DJNZ	t0, :sram_to_ram

	SRAM_OFF			; Disable the SRAM
	
	; Load the next destination SRAM page to write
	_BANK	BANK_STACK
	MOV	t1, s9			; t1:t0 points to the base of the page
	MOV	t0, s8
	CALL	@__SRAM_Load_Page_W

	; Copy the RAM buffer to the SRAM page
	MOV	t0, #5			; Unroll loop 3x, copy 15 bytes
	CLR	FSR
	_BANK	BANK_SRAM_COPY_BUFF
:ram_to_sram

	MOV	RD, IND			; 0
	INC	FSR
	INC	RC
	NOP
	NOP
	MOV	RD, IND			; 1
	INC	FSR
	INC	RC
	NOP
	NOP
	MOV	RD, IND			; 2
	INC	FSR
	INC	RC

	DJNZ	t0, :ram_to_sram

	; Copy the last byte outside of the loop
	MOV	RD, IND
	NOP
	NOP

	SRAM_OFF			; Disable the SRAM

	; Increment the source and destination addresses
	_BANK	BANK_STACK
	ADD	s6, #1
	ADDB	s7, C
	ADD	s8, #1
	ADDB	s9, C

	; Copy next page
	DJNZ	t5, @:copy_page

	; Restore the row counter
	MOV	t0, s0

	; Skip past the alignment page
	_BANK	BANK_STACK
	ADD	s6, #1
	ADDB	s7, C
	ADD	s8, #1
	ADDB	s9, C

	; Copy next row
	DJNZ	t0, @:copy_row
ENDASM
	RETURN



' *****************************************************************************
' *
' *	Read_Joysticks (@stick1{,@stick2})
' *
' *	Reads in the status of both joysticks.
' *

Read_Joysticks:
  ASM
	MOV t5,__PARAMCNT
	JOY_PORT_MASK		equ	%00000111 ; mask for bits used by joystick interface from SX52

	; Set the RA port's pin directions for joystick interfacing
 	_MODE	#$1F
	MOV	RA,  #%00000000
	MOV	!RA, #%11111100

	; step 1: read in port and mask control bits
	mov	W, RA
	and	W, #JOY_PORT_MASK

	; step 2: prepare for read

	mov	RA, W

	; step 3: latch joysticks into shift registers

	clrb	RA.1		; JOY_SH/LD = (0), set parallel load mode

	DELAY(1)

	setb	RA.0		; JOY_CLK   = (1), clock

	DELAY(1)

	clrb	RA.0		; JOY_CLK   = (0), clock

	DELAY(1)

	; step 4: shift data into system, 16-bits

	setb	RA.1		; JOY_SH/LD = (1), set serial shift mode

	; shift 16-bits of address into latch
	mov 	t4, #16	; 16 bits per joystick read

:Read_Joy_Bit_Loop

	rl	t2		; rotate results right thru carry
	rl	t3		; rotate upper results from from including carry

	; read joy in data on port bit first

	sb 	RA.2		; jump over if set
	jmp 	:Joy_Bit_Zero

	; bit set, write 1 to joystick results packet

	setb 	t2.0	; data8[8] = (1)
	jmp	:Joy_Clock_Next_Bit

:Joy_Bit_Zero

	; bit clear, write 0 to joystick results packet

	clrb 	t2.0	; data8[7] = (0)

:Joy_Clock_Next_Bit

	; clock next data bit

	DELAY(1)

	setb	RA.0		; JOY_CLK = (1), clock

	DELAY(1)

	clrb	RA.0		; JOY_CLK = (0), clock

	DELAY(1)
		
	djnz 	t4, :Read_Joy_Bit_Loop

	; reset all joystick control bits
	mov	W, RA
	and	W, #JOY_PORT_MASK
	mov	RA, W

	; Restore the pin directions on RA
 	_MODE	#$1F
	MOV	RA,  #%00000000
	MOV	!RA, #%00000000

  ENDASM
  IF t5 = 2 THEN
	__RAM(__PARAM2)=t3
  ENDIF
  __RAM(__PARAM1)=t2
  RETURN


' *****************************************************************************
' *
' *	Write_Sound_Cmd (Register, Value)
' *
' *	Writes a command to the sound chip.

Write_Sound_Cmd:

	' __PARAM1 = screg
	' __PARAM2 = scdata

	RA = 0
	PAUSEUS 1
	RA.SSTB_BIT = 1
	PAUSEUS 1
	RA.SSTB_BIT = 0

	FOR t5 = 1 TO 11		' 11 bits per command
		RA.SCLK_BIT = 0		' SCLK = 0
		RA.SDATA_BIT = __PARAM1.2
		PAUSEUS 1
		RA.SCLK_BIT = 1		' SCLK = 1
		PAUSEUS 1
		\RL	__PARAM2
		\RL	__PARAM1
	NEXT 
	RETURN

' End Write_Sound_Cmd

' *****************************************************************************
' *
' *	Play_Sound(Waveform, Freq)
' *
' *	Plays a sound.

Play_Sound:

	' Safely copy the parameters
	t3 = __PARAM1	' Waveform type
	t4 = __PARAM2	' Frequency

	' COMMAND 1: Turn on the channel and set its properties
	t5 = SND_OPER_WAVE1_ON OR SND_OPER_TNSEL_MELODY
	t5 = t5 OR SND_OPER_FSEL_MELODY
	t5 = t5 OR SND_OPER_SLEEP_OFF
	Write_Sound_Cmd SND_REG_OPER, t5

	' COMMAND 2: Set additional channel properties
        t5 = SND_MODE_WMODE_WAVE1 OR t3
	t5 = t5 OR SND_MODE_ENVON_ON
	t5 = t5 OR SND_MODE_ENVM_FAST
	t5 = t5 OR SND_MODE_ATTN_0_0DB
	Write_Sound_Cmd SND_REG_MODE, t5

	' COMMAND 3: Set the frequency 
	t5 = SND_FREQ_WAVE1 OR t4
	Write_Sound_Cmd SND_REG_FREQ, t5

	' COMMAND 4: Set the master volume
	Write_Sound_Cmd SND_REG_GAIN, SND_GAIN_VOLUME_0DB

	RETURN

' End Play_Sound